Package: bigmelon
Type: Package
Title: Illumina methylation array analysis for large experiments
Version: 1.37.0
Authors@R: c(
    person(c("Tyler", "J."), "Gorrie-Stone", role="aut"),
    person("Ayden", "Saffari", role="aut"),
    person("Karim", "Malki", role="aut"),
    person(c("Leonard", "C."), "Schalkwyk", role=c("cre", "aut"), email="lschal@essex.ac.uk")
    )
Description: Methods for working with Illumina arrays using gdsfmt.
License: GPL-3
Depends: R (>= 3.3), wateRmelon (>= 1.25.0), gdsfmt (>= 1.0.4),
        methods, minfi (>= 1.21.0), Biobase, methylumi
Imports: stats, utils, GEOquery, graphics, BiocGenerics, illuminaio
Suggests: BiocGenerics, RUnit, BiocStyle, minfiData, parallel,
        IlluminaHumanMethylation450kanno.ilmn12.hg19,
        IlluminaHumanMethylationEPICanno.ilm10b2.hg19, bumphunter
LazyLoad: yes
Collate: zzz.R es2gds.R dasenGds.R dbGdsn.R dfsfitGdsn.R gds2mlumi.R
        gdsnclass_methods.R inout.R pfilterGds.R prcompGdsn.R
        pwodGdsn.R qnGdsn.R comboGds.R ranknorm.R GEOtoGDS.R ecc.gdsn.R
        bumphunterGdsn.R chainsaw.R
biocViews: DNAMethylation, Microarray, TwoChannel, Preprocessing,
        QualityControl, MethylationArray, DataImport, CpGIsland
NeedsCompilation: no
RoxygenNote: 7.3.1
git_url: https://git.bioconductor.org/packages/bigmelon
git_branch: devel
git_last_commit: 142a0e8
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
Packaged: 2025-11-04 21:25:46 UTC; biocbuild
Author: Tyler J. Gorrie-Stone [aut],
  Ayden Saffari [aut],
  Karim Malki [aut],
  Leonard C. Schalkwyk [cre, aut]
Maintainer: Leonard C. Schalkwyk <lschal@essex.ac.uk>
