#TODO - unit tests for non-numeric accessors, & all normalisation methods

# unit test stubs, the  idea is that whenever we work on a function
# we create a unit test. any function for strictly internal use should be moved
# to test_internal.R

# TODO: dasenGds.R dbGdsn.R dfsfitGdsn.R
#       ecc.gdsn.R es2gds.R gds2mlumi.R gdsnclass_methods.R
#       GEOtoGDS.R inout.R pfilterGds.R prcompGdsn.R pwodGdsn.R
#       qnGdsn.R ranknorm.R zzz.R

# comboGds.R
